import yfinance as yf
import wansuite as ws
import numpy as np
import pandas as pd
import wansuite as ws
pd.options.plotting.backend = "plotly"
import matplotlib.pyplot as plt
import seaborn as sns
from datetime import timedelta
import scipy.stats as stats
from sklearn import datasets, ensemble
class ZackStrategy:
    def __init__(self,sector,startdate,lookback_window,seed):
        self.lookback_window = lookback_window
        self.sector=sector
        self.startdate=startdate
        self.zack=None
        self.dailymarket=None
        self.featurelist=[]
        self.hfmarket=None
        self.ind=None
        self.rank=None
        self.wealth=None
        self.trainsize=None
        self.traindates=None
        self.metric=None
        self.tempx=None
        self.tempy=None
        self.horizon=None
        self.randomseed=seed
        self.performance=None
        self.performancelist=[]
        self.time_1=None
        self.time_2=None
        self.ln=None
        self.sn=None
        self.look=None
        self.looks=None
        self.temp=None
        self.temp1=None
        self.temp2=None
        self.perlist=[]
        self.long=None
        self.short=None
        self.firmsize=None
        self.params = {
            "n_estimators": 300,
            "max_depth": 3,
            "min_samples_split": 5,
            "learning_rate": 0.2,
            "loss": "squared_error",
        }
        self.model = ensemble.GradientBoostingRegressor(**self.params, random_state=self.randomseed)
        self.modellist=[]

    def backtest_trend_hf(self, wealth, size, duration):
        self.wealth = wealth
        self.trainsize = size
        self.duration = duration
        self.perlist=[]
        for t in self.ind[self.trainsize:]:
            traindates = self.zack[self.zack.Today <= t].Today.value_counts().sort_index().iloc[-self.trainsize:].index

            data2 = self.zack.loc[(self.zack.Today.isin(traindates)) & (self.zack.MarketCap_mil > 3000)].dropna()

            # data2=data2[data2.Price>10]

            tempx = data2[self.featurelist]
            tempy = data2["Y"]
            self.tempx = tempx
            self.tempy = tempy

            # tempx=pd.concat([data2[featurelist],data2[featurelist]])
            # tempy=pd.concat([data2["Y"],data2["Y1"]])
            self.model.fit(tempx, tempy)

            performance = data2.copy()

            performance["Predict"] = self.model.predict(performance[self.featurelist])
            temp = performance[["Ticker", "MarketCap_mil", "Predict", "Price"]].groupby("Ticker").mean().sort_values(
                by="Predict")
            temp["ID"] = np.arange(len(temp))

            self.rank.loc[t, temp.index] = temp.loc[:, "ID"]

            time_1 = t + timedelta(days=1)
            time_2 = t - timedelta(days=4)
            self.time_1 = time_1
            self.time_2 = time_2

            # print(temp0[temp0.Ticker.isin(["NUVL","MORF","MRTX", "KYMR","OBIO", "KYMR"])][["Ticker"]])
            # temp=temp[temp.Price>10]

            if "ATHM" in temp.index:
                temp = temp.drop("ATHM")
            if "HZNP" in temp.index:
                temp = temp.drop("HZNP")
            if "PNT" in temp.index:
                temp = temp.drop("PNT")
            if "SMCI" in temp.index:
                temp = temp.drop("SMCI")

                # if "BHVN" in temp.index:
            #   temp=temp.drop("BHVN")
            temp1 = temp[temp.MarketCap_mil > 3000][temp.MarketCap_mil < 10000]
            temp2 = temp[temp.MarketCap_mil > 10000]

            # if len(temp1) > 0 and len(temp2) > 0 and temp1.iloc[-2]["Predict"] > 0 and temp2.iloc[1]["Predict"] < 0:
            if len(temp1) > 0 and len(temp2) > 0:
                try:

                    ln = temp1.index[-2]
                    sn = temp2.index[1]
                    # print(temp1.shape, temp2.shape, ln, sn)
                    look = self.hfmarket.loc[(self.hfmarket.index == ln) & (self.hfmarket.Date >= time_1.date())].iloc[
                           0:self.duration].set_index(
                        "Datetime")
                    # print(look)
                    looks = self.hfmarket.loc[(self.hfmarket.index == sn) & (self.hfmarket.Date >= time_1.date())].iloc[
                            0:self.duration].set_index(
                        "Datetime")
                    look["diff"]= look["Open"].shift(-1)-look["Open"]
                    look["fast"]= look["Open"].rolling(5).mean()
                    look["slow"] =  look["Open"].rolling(15).mean()
                    look["signal"]=[1 if look.loc[t,"fast"]>look.loc[t,"slow"] else np.nan for t in look.index]
                    longprofit=(wealth[0]/look["Open"].iloc[0])*(look["diff"]*look["signal"]).sum()
                    looks["diff"] = looks["Open"].shift(-1) - looks["Open"]
                    looks["fast"] = looks["Open"].rolling(2).mean()
                    looks["slow"] = looks["Open"].rolling(10).mean()
                    looks["signal"] = [-1 if looks.loc[t, "fast"] < looks.loc[t, "slow"] else np.nan for t in looks.index]
                    shortprofit = (wealth[1] / looks["Open"].iloc[0]) * (looks["diff"] * looks["signal"]).sum()
                    self.long=look
                    self.short=looks
                    self.perlist.append([time_1, ln, sn,longprofit,shortprofit])
                    print(ln,sn,perlist[-1][:-2:])
                except:
                    pass
            self.performance = pd.DataFrame(self.perlist,
                                            columns=["date", "longstock", "shortstock", "longreturn", "shortreturn"])

            self.performance.set_index("date", inplace=True)
            self.performance.sort_index(inplace=True)
            self.performance["total"] = self.performance["longreturn"] + self.performance["shortreturn"]
            print(time_1, temp1.shape, temp2.shape, ln, sn)

    def train(self,  size,firmsize):

        self.trainsize = size
        self.firmsize=firmsize
        for t in self.ind[self.trainsize:]:
            self.traindates = self.zack[self.zack.Today <= t].Today.value_counts().sort_index().iloc[
                              -self.trainsize:].index
            self.performance = self.zack.loc[
                (self.zack.Today.isin(self.traindates)) & (self.zack.MarketCap_mil > self.firmsize)]
            self.performance=self.performance.loc[:,self.performance.isnull().sum()<50].dropna()


            self.tempx = self.performance[self.featurelist]
            self.tempy = self.performance["Y"]
            self.model.fit(self.tempx, self.tempy)
            self.performance["Predict"] = self.model.predict(self.performance[self.featurelist])
            self.performancelist.append(self.performance)



    def backtest(self,wealth,duration,longsize,shortsize):
        self.wealth=wealth
        self.duration=duration
        self.perlist=[]

        for n in range(len(self.ind[self.trainsize:])):
            t=self.ind[self.trainsize:][n]
            self.traindates = self.zack[ self.zack.Today <= t].Today.value_counts().sort_index().iloc[-self.trainsize:].index

            self.performance = self.performancelist[n]

            self.temp = self.performance[["Ticker", "MarketCap_mil", 'FEG1','FEG2',"Predict", "Price"]].groupby("Ticker").mean().sort_values(
                by="Predict")

            self.time_1=t + timedelta(days=1)
            self.time_2=t - timedelta(days=4)

            self.temp1 = self.temp[self.temp.MarketCap_mil >longsize][self.temp.MarketCap_mil <shortsize]
            self.temp2 = self.temp[self.temp.MarketCap_mil >shortsize]


            #if len(temp1) > 0 and len(temp2) > 0 and temp1.iloc[-2]["Predict"] > 0 and temp2.iloc[1]["Predict"] < 0:
            if len(self.temp1) > 0 and len(self.temp2) > 0:
                try:
                    print("start")
                    self.ln =self.temp1.index[1]
                    self.sn =self.temp2.index[1]
                   # print(temp1.shape, temp2.shape, ln, sn)
                    self.look = self.dailymarket.loc[(self.dailymarket.index == self.ln) & (self.dailymarket.Date.dt.date >= self.time_1.date())].iloc[0:self.duration].set_index(
                        "Date")
                    #print(look)
                    self.looks = self.dailymarket.loc[(self.dailymarket.index == self.sn) & (self.dailymarket.Date.dt.date >= self.time_1.date())].iloc[0:self.duration].set_index(
                        "Date")
                    self.perlist.append([self.time_1,self.ln,self.sn, self.wealth[0] * (self.look.iloc[-1]["Close"] - self.look.iloc[0]["Open"]) / (self.look.iloc[0]["Open"]), self.wealth[1]* (
                                self.looks.iloc[-1]["Close"] - self.looks.iloc[0]["Open"]) / (self.looks.iloc[0]["Open"])])
                    print(self.ln,self.sn,self.perlist[-1])

                except:
                    pass
            self.metric=pd.DataFrame(self.perlist,columns=["date","longstock","shortstock","longreturn","shortreturn"])

            self.metric.set_index("date",inplace=True)
            self.metric.sort_index(inplace=True)
            self.metric["total"] = self.metric["longreturn"]+self.metric["shortreturn"]
            #print(time_1,temp1.shape, temp2.shape, ln, sn)
    def backtest_trend(self,wealth,duration,longsize,shortsize):
        self.wealth=wealth
        self.duration=duration
        self.perlist=[]
        for n in range(len(self.ind[self.trainsize:])):
            t=self.ind[self.trainsize:][n]
            self.traindates = self.zack[ self.zack.Today <= t].Today.value_counts().sort_index().iloc[-self.trainsize:].index

            self.performance = self.performancelist[n]

            self.temp = self.performance[["Ticker", "MarketCap_mil", "Predict", "Price"]].groupby("Ticker").mean().sort_values(
                by="Predict")

            self.time_1=t + timedelta(days=1)
            self.time_2=t - timedelta(days=4)

            self.temp1 = self.temp[self.temp.MarketCap_mil >longsize]
            self.temp2 = self.temp[self.temp.MarketCap_mil >shortsize]


            #if len(temp1) > 0 and len(temp2) > 0 and temp1.iloc[-2]["Predict"] > 0 and temp2.iloc[1]["Predict"] < 0:
            if len(self.temp1) > 0 and len(self.temp2) > 0:
                try:
                    print("start")
                    self.ln = self.temp1.index[-5]
                    self.sn =self.temp2.index[1]
                   # print(temp1.shape, temp2.shape, ln, sn)
                    self.look = self.dailymarket.loc[(self.dailymarket.index == self.ln) & (self.dailymarket.Date.dt.date >= self.time_1.date())].iloc[0:self.duration].set_index(
                        "Date")
                    #print(look)
                    self.looks = self.dailymarket.loc[(self.dailymarket.index == self.sn) & (self.dailymarket.Date.dt.date >= self.time_1.date())].iloc[0:self.duration].set_index(
                        "Date")

                    self.look["signal"] = [1 if self.look.loc[t, "fast"] > self.look.loc[t, "slow"] else np.nan for t in self.look.index]
                    longprofit = (self.wealth[0] / self.look["Open"].iloc[0]) * (self.look["future"] * selflook["signal"]).sum()

                    self.looks["signal"] = [-1 if self.looks.loc[t, "fast"] < self.looks.loc[t, "slow"] else np.nan for t in
                                       self.looks.index]
                    shortprofit = (self.wealth[1] / self.looks["Open"].iloc[0]) * (self.looks["future"] * self.looks["signal"]).sum()

                    self.perlist.append([self.time_1,self.ln,self.sn, longprofit,shortprofit ])
                    print(self.ln,self.sn,self.perlist[-1])

                except:
                    pass
            self.metric=pd.DataFrame(self.perlist,columns=["date","longstock","shortstock","longreturn","shortreturn"])

            self.metric.set_index("date",inplace=True)
            self.metric.sort_index(inplace=True)
            self.metric["total"] = self.metric["longreturn"]+self.metric["shortreturn"]
            #print(time_1,temp1.shape, temp2.shape, ln, sn)
    def PrepapreIndustry(self):
        data= self.zack.copy()
        data.Today = [x.date() for x in self.zack.Today]

        for l in ['Price', 'MarketCap_mil', 'SharesOutstanding_mil',
                  'AvgVolume', 'Price_Percentof52WkH-LRange', 'Percent_PriceChange_1Week',
                  'Percent_PriceChange_4Weeks', 'Percent_PriceChange_12Weeks',
                  'AverageTargetPrice', 'Percent_RatingDowngrades',
                  'Percent_RatingUpgrades', 'Percent_Rating_Hold',
                  'Percent_RatingStrongSellorSell', 'Percent_Rating_StrongBuyorBuy',
                  'Q0EPS', 'Q1EPS', 'Q2EPS', 'F0EPS', 'F1EPS', 'F2EPS', 'CurrentROE_TTM',
                  'CurrentROI_TTM', 'ROI_ 5YrAvg', 'CurrentROA_TTM', 'ROA_5YrAvg',
                  'MarketValue/Analysts', 'EBITDA_mil', 'EBIT_mil', 'NetIncome_mil',
                  'CashFlow_mil', 'NetIncomeGrowthF0/F-1', 'NetMargin_Percent',
                  'Turnover', 'InventoryTurnover', 'AssetUtilization',
                  'OperatingMargin_12Mo_Percent', 'Receivables_mil', 'Inventory_mil',
                  'Intangibles_mil', 'CurrentAssets_mil', 'CurrentLiabilities_mil',
                  'LongTermDebt_mil', 'PreferredEquity_mil', 'CommonEquity_mil',
                  'BookValue', 'Debt/TotalCapital', 'Debt/EquityRatio', 'CurrentRatio',
                  'QuickRatio', 'CashRatio']:
            self.zack[l] = [np.nan if x == "NULL" else float(x) for x in self.zack[l]]

        self.featurelist.append('Percent_PriceChange_12Weeks')
        self.featurelist.append('Percent_PriceChange_4Weeks')

        self.zack["PercentTarget"] = (self.zack["Price"] - self.zack['AverageTargetPrice']) / (self.zack['AverageTargetPrice'] + 0.0001)
        self.featurelist.append("PercentTarget")
        self.featurelist.append("QEG1")
        self.featurelist.append("QEG2")
        self.zack["QEG1"] = (self.zack["Q1EPS"] - self.zack["Q0EPS"]) / (self.zack["Q0EPS"] + 0.0001)
        self.zack["QEG2"] = (self.zack["Q2EPS"] - self.zack["Q1EPS"]) / (self.zack["Q1EPS"] + 0.0001)
        self.featurelist.append("FEG1")
        self.featurelist.append("FEG2")
        self.zack["FEG1"] = (self.zack["F1EPS"] - self.zack["F0EPS"]) / (self.zack["F0EPS"] + 0.0001)
        self.zack["FEG2"] = (self.zack["F2EPS"] -self.zack["F1EPS"]) / (self.zack["F1EPS"] + 0.0001)
        self.featurelist.append("FPE1")
        self.featurelist.append("FPE2")
        self.zack["FPE1"] = self.zack["Price"] / (self.zack["F1EPS"] + 0.0001)
        self.zack["FPE2"] = self.zack["Price"] / (self.zack["F2EPS"] + 0.0001)
        self.featurelist.append("FPEG1")
        self.featurelist.append("FPEG2")
        self.zack["FPEG1"] = self.zack["Price"] / (self.zack["FEG1"] + 0.0001)
        self.zack["FPEG2"] = self.zack["Price"] / (self.zack["FEG2"] + 0.0001)
        self.featurelist.append("QPE1")
        self.featurelist.append("QPE2")
        self.zack["QPE1"] =self.zack["Price"] / (self.zack["Q1EPS"] + 0.0001)
        self.zack["QPE2"] = self.zack["Price"] / (self.zack["Q2EPS"] + 0.0001)
        self.featurelist.append("QPEG1")
        self.featurelist.append("QPEG2")
        self.zack["QPEG1"] =self.zack["Price"] / (self.zack["QEG1"] + 0.0001)
        self.zack["QPEG2"] =self.zack["Price"] / (self.zack["QEG2"] + 0.0001)
        self.featurelist.append('CurrentROE_TTM')
        self.featurelist.append('CurrentROI_TTM')
        self.featurelist.append('CurrentROA_TTM')
        self.featurelist.append('EBITDA_mil')
        self.featurelist.append('EBIT_mil')
        self.featurelist.append('NetIncomeGrowthF0/F-1')
        self.cl=None
        for x in ['Debt/TotalCapital', 'Debt/EquityRatio', 'CurrentRatio']:
            self.featurelist.append(x)
            # data.columns


    def getZack(self):
        df = ws.msql.table2df("macro", "zack")
        df = df[df["Sector"] == self.sector]
        df = df.drop_duplicates(subset=["Today", "Ticker"], keep="last")
        df.Today.value_counts().sort_index()
        df["Price"] = [float(x) for x in df["Price"]]
        df = df[df["Sector"] ==self.sector]
        df["Today"] = pd.to_datetime(df["Today"])
        df = df.drop_duplicates(subset=["Today", "Ticker"])
        cl = df.Ticker.value_counts().index
        df = df.sort_values(by=["Today"])
        df = df.set_index("Ticker")
        df["Y"]=np.nan
        self.zack=df

    def getDailyStock(self,Trend=False):
        if self.sector=="Computer and Technology":
            market = ws.msql.table2df("market", "yf_computer_1d")
        elif self.sector=="Medical":
            market = ws.msql.table2df("market", "yf_health_1d")

        elif self.sector=="Finance":
            market = ws.msql.table2df("market", "yf_finance_1d")
        elif self.sector == "Oils-Energy":
            market = ws.msql.table2df("market", "yf_energy_1d")
        elif self.sector == "Business Services":
            market = ws.msql.table2df("market", "yf_service_1d")
        elif self.sector == "Industrial Products":
            market = ws.msql.table2df("market", "yf_products_1d")
        elif self.sector == "Consumer Discretionary":
            market = ws.msql.table2df("market", "yf_discretionary_1d")

        market["Date"] = pd.to_datetime(market["Date"])
        market = market.drop_duplicates(subset=["Date", "Symbol"])
        market = market.set_index("Symbol")
        market = market.sort_values(by=["Date"])
        if Trend==True:
            market["fast"]=np.nan
            market["slow"]=np.nan
            for c in market.index.value_counts().index:
                market.loc[market.index == c, "fast"]=market.loc[market.index == c, "Open"].rolling(5).mean()
                market.loc[market.index == c, "slow"]=market.loc[market.index == c, "Open"].rolling(20).mean()
                market.loc[market.index == c, "future"]=market.loc[market.index == c, "Open"].shift(-1)-market.loc[market.index == c, "Open"]
        self.dailymarket=market

    def prepareTrain(self):
        #self.zack=self.zack.set_index("Ticker")
        cl = set(self.zack.index.value_counts().index).intersection(self.dailymarket.index.value_counts().index)
        #print(self.zack.index.value_counts().index)
        #print(self.dailymarket.index.value_counts().index)
        #print(cl)
        self.cl=cl
        for c in cl:
            self.dailymarket.loc[c, "Y"] = self.dailymarket.loc[c, "Open"].pct_change(self.lookback_window)
            commondate = list(set(self.dailymarket.loc[c, "Date"]).intersection(set(self.zack.loc[c, "Today"])))
            self.zack.loc[(self.zack.index == c) & (self.zack.Today.isin(commondate)), "Y"] = self.dailymarket.loc[
                (self.dailymarket.index == c) & self.dailymarket.Date.isin(commondate), "Y"]

        self.PrepapreIndustry()
        self.zack =self.zack.reset_index()
        self.ind=self.zack.Today.value_counts().sort_index().index
        self.rank = pd.DataFrame(index=self.ind, columns=self.zack.Ticker.value_counts().index)



    def handleMissingValue(self):
        dl = self.zack[ self.zack["Today"] == pd.to_datetime( self.zack["Today"].iloc[-1]).date()].isnull().sum().sort_values().tail(
            3).index
        columns = list( self.zack.columns)
        for n in dl:
            if n in columns:
                columns.remove(n)
            if n in self.featurelist:
                self.featurelist.remove(n)
        self.zack = self.zack[columns]
        self.zack = self.zack.loc[self.zack[self.featurelist].dropna().index]
        self.zack = self.zack.dropna(subset=["Y"])


    def getMinuteData(self):
        if self.sector=="Computer and Technology":
            self.hfmarket = ws.msql.table2df("market", "yf_computer_5m")
        elif self.sector == "Medical":
            self.hfmarket = ws.msql.table2df("market", "yf_health_5m")
        elif self.sector == "Finance":
            self.hfmarket = ws.msql.table2df("market", "yf_finance_5m")
        elif self.sector == "Oils-Energy":
            self.hfmarket = ws.msql.table2df("market", "yf_energy_5m")

        elif self.sector == "Business Services":
            market = ws.msql.table2df("market", "yf_service_5m")
        elif self.sector == "Industrial Products":
            market = ws.msql.table2df("market", "yf_products_5m")
        self.hfmarket["Datetime"] = pd.to_datetime(self.hfmarket["Datetime"])
        self.hfmarket = self.hfmarket.sort_values(by="Datetime")
        self.hfmarket["Date"] = [x.date() for x in self.hfmarket.Datetime]
        self.hfmarket = self.hfmarket.loc[self.hfmarket.Date > self.zack.Today.iloc[0].date()]
        self.hfmarket = self.hfmarket.set_index("Symbol")



import pandas as pd
import json
import numpy as np
import matplotlib.pyplot as plt
import plotly.express as px
import plotly.graph_objects as go
import warnings
warnings.filterwarnings('ignore')
from plotly.subplots import make_subplots
pd.options.plotting.backend = "plotly"


from wordcloud import WordCloud
import pyLDAvis
from pyLDAvis import gensim_models
import plotly.express as px

def LDA_graph(model,work_frequency,dictionary, notebook_enable=True):
    if notebook_enable==True:
        pyLDAvis.enable_notebook()
    return gensim_models.prepare(model,words_frequency,dictionary=dictionary)

def word_cloud(wordlist,width=1000, height=500, scheme="plotly_dark",margin=0, collocations=False):
    if wordlist[0][1]=="#":
        wc = " "
        for x in wordlist:
            wc = wc + x[1:] + " "
    else:
        wc = " "
        for x in wordlist:
            wc = wc + x[0:] + " "
    cloud=WordCloud(background_color="white", width=width, height=height, margin=margin, collocations=collocations).generate(wc)
    fig = px.imshow(cloud)
    fig.update_layout(
        title='',
        width=width,
        height=height, template=scheme
    )

    fig.show()



#%%
import sqlite3
import time
from bs4 import BeautifulSoup
from helium import *
import pandas as pd
import wansuite as ws
from sqlalchemy import create_engine, Table, Column, Integer, String, MetaData, Date, DateTime, Time, Float
from wansuite import msql, investing
import numpy as np
import re
#%%
ccount = 0
start = False
newspages = {}
newspages["economy"] = "https://www.investing.com/news/economy"
newspages["stock"] = "https://www.investing.com/news/stock-market-news"
newspages["commodity"] = "https://www.investing.com/news/commodities-news"
newspages["currency"] = "https://www.investing.com/news/forex-news"
newspages["indicator"] = "https://www.investing.com/news/economic-indicators"
newspages["world"] = "https://www.investing.com/news/world-news"
newspages["politics"] = "https://www.investing.com/news/politics"
#%%
for m in newspages.keys():
    links = ["https://www.investing.com/news/stock-market-news/a-second-kobayashi-pharma-japan-factory-inspected-over-deaths-3358582"]
    print(m)
    for t in range(1, 2, 1):  # 200
        url = newspages[m] + "/" + str(t)
        print(url)
        start_chrome(url, headless=True)
        print(t)
        # Assuming `news` is a Selenium WebElement containing the HTML of the news article container
        soup = BeautifulSoup(news.get_attribute('innerHTML'), 'html.parser')

        # Use the correct CSS selector to find the <a> tag with the article link
        for link in soup.select('a[data-test="article-title-link"]'):
            links.append(link.get('href'))

        for news in find_all(S('.largeTitle')):
            print(news)
            try:
                soup = BeautifulSoup(news.web_element.get_attribute('innerHTML'), 'html.parser')
                for link in soup.find_all('a'):
                    links.append(link.get('href'))
            except:
                # Element went stale. Retry the operation.
                continue
        kill_browser()
        print(links)
    links = list(set(links))
    links = [x for x in links if x[-8:] != "comments"]
    print("need to update", n, len(links))
    for k in links:
        df = pd.DataFrame(columns=["datetime", "title", "article", "link", "label", "domain"])
        lin = k

        try:
            url0 = "investing.com" + lin
            print(url0)
            start_chrome(url0, headless=True)
            left_columns_html = get_driver().execute_script('return document.getElementById("leftColumn").innerHTML')
            soup = BeautifulSoup(left_columns_html, 'html.parser')

            # Extract the span, p, and h1 elements
            spans = soup.find_all('span')
            ps = soup.find_all('p')
            h1s = soup.find_all('h1', class_='articleHeader')
            article = soup.find_all('div', class_='WYSIWYG articlePage')
            kill_browser()
            cleanarticle = []
            for ar in article:
                cleanarticle.append(ar.get_text())
            news = cleanarticle[0].split("\n\n")[-1]

            title = ""
            author = ""
            resource = ""
            article = ""
            domain=m

            datetime = ""
            label = ""
            cleanspans = []
            for p in spans:
                cleanspans.append(p.get_text())
            for hh in cleanspans:
                if "Published" in hh:
                    datetime = hh
                    break
            if datetime == "":
                spans = soup.find_all('span')
                for p in spans:
                    cleanspans.append(p.get_text())
                for hh in cleanspans:
                    if "Published" in hh:
                        datetime = hh
                        break

            #             temp=re.findall(r'\((.*?)\)', cleanspans[0])
            #             if len(temp)==0:
            #                 datetime= cleanspans[0]
            #             else:
            #                  datetime=temp[-1]

            for h1 in h1s:
                title = h1.get_text()

            cleanps = []
            for p in ps:
                cleanps.append(p.get_text())

            df.loc[len(df)] = [datetime, title, news, url, label,domain]
            if ccount == 0:
                msql.df2table("media", df, "investing_com_news", "append")
                ccount = ccount + 1


        except:
            print("cannot get article", m, " ", lin)

            pass
        try:
            kill_browser()
        except:
            pass




import numpy as np
import pandas as pd

def getlink(shareurl):
  return 'https://docs.google.com/uc?export=download&id='+shareurl.split('/')[-2]

def econnews():
  url = getlink("https://drive.google.com/file/d/1nfuhwK_hjqPsfbp7ybXgZOUWZkgPqyDe/view?usp=drive_link")
  return  pd.read_csv(url, index_col=0).drop_duplicates()


def stocknews():
  url = getlink("https://drive.google.com/file/d/1WZyAEmqSf0Lskx4_-qJq3rz4PEdGkQVA/view?usp=drive_link")
  return  pd.read_csv(url, index_col=0).drop_duplicates()

def yelprating():
  url = getlink("https://drive.google.com/file/d/1KR4rA1AlteQv1QEKHbWkdKhTRHcrr6oD/view?usp=drive_link")
  return  pd.read_csv(url, index_col=0).drop_duplicates()

def twitter():
  url = getlink("://drive.google.com/file/d/1frox8TLvzsK94rSu43dBVrwpvu8JL9Si/view?usp=sharing")
  return  pd.read_csv(url, index_col=0).drop_duplicates()

def eurusd():
  url = getlink("https://drive.google.com/file/d/1YJTJyklVqWKbHLVCUez-VMm3msOq-Dse/view?usp=share_link")
  return  pd.read_csv(url, index_col=0).drop_duplicates()

def forex():
  url = 'https://drive.google.com/uc?id=1Si7fsfE1rdA1xHOFF_vsA3tvFkQAYx-2'
  return  pd.read_csv(url, index_col=0).drop_duplicates()

def google():
  url = 'https://docs.google.com/uc?export=download&id=1fkteL4yNpnBsozMy0FtVRA2-r6hYkkiV'
  return  pd.read_csv(url, index_col=0).drop_duplicates()

def twitter116():
  url = 'https://docs.google.com/uc?export=download&id=1fi-Xf8obJzJ5JPwrvyMhxlKIK48SpU0F'
  return  pd.read_csv(url, index_col=0).drop_duplicates()

def keyword():
  url = 'https://docs.google.com/uc?export=download&id=1fQCoNqFzmKRDcOoYGJnwqi2deklh5FJl'
  return  pd.read_csv(url, index_col=0).drop_duplicates()

def news2023():
  url = 'https://docs.google.com/uc?export=download&id=1jHNpNJylqSScWnF9FIlS0dvoWEoqgDcp'
  return  pd.read_csv(url, index_col=0).drop_duplicates()

def inflation():
  url = getlink("https://drive.google.com/file/d/1a92uW0jmbpunugF-ZyzL2fN7fn25AU0C/view?usp=sharing")
  return  pd.read_csv(url, index_col=0).drop_duplicates()


def twitter_gpt():
  url = getlink("https://drive.google.com/file/d/1oJZiII2GiIZNpMm5jI1qKCJVZG_TyBdy/view?usp=drive_link")
  return  pd.read_csv(url, index_col=0).drop_duplicates()


from gensim import models
from gensim.models.coherencemodel import CoherenceModel
def LDA_Model(words_frequency,dictionary,num_topics=50, alpha=0.5, beta=0.5,passes=20,random_state = 100):
    return models.ldamodel.LdaModel(words_frequency, num_topics=num_topics, id2word =dictionary,passes=passes,alpha=alpha,eta=beta,random_state = random_state)

def coherence_measure_lda(model,words_frequency,coherence='u_mass'):
    return CoherenceModel(model=model, corpus=words_frequency, coherence=coherence)




from . import network
from . import sentiment
from . import topicmodel
from . import textprocessing
from . import datavis
from . import sys
from . import datacollection
from . import network
from . import textfeature
from . import nownews


import re
import nltk
from nltk.tokenize import RegexpTokenizer
from stop_words import get_stop_words
from nltk.corpus import stopwords
from nltk.stem import PorterStemmer
from nltk.tokenize import word_tokenize
nltk.download('stopwords')
nltk.download('punkt')
from gensim import corpora
stop_words = set(stopwords.words('english'))
en_stop = get_stop_words('en')
def tokenize(x,sep=r"\w+"):
	tokenizer = RegexpTokenizer(sep)
	return tokenizer.tokenize(x.lower())

def stopwords(x):

	return [y for y in x if y not in en_stop]

def stem(x):
	p_stemmer = PorterStemmer()
	return [p_stemmer.stem(y) for y in x]

def dictionary(collectiion):
	Vocabulary = []
	for x in collectiion:
		Vocabulary.extend(x)
	Vocabulary = list(set(Vocabulary))
	dictionary = corpora.Dictionary(collectiion)
	dictlist = list(dictionary.token2id.items())
	ID_Word = {}
	for it in dictlist:
		ID_Word[it[1]] = it[0]
	return Vocabulary,dictionary,ID_Word

def words_frequency (collectiion,dictionary):
	return  [dictionary.doc2bow(d) for d in collectiion]

def preprocess_text(text):
    # Tokenize
    tokens = word_tokenize(text.lower())

    # Remove stopwords
    stop_words =en_stop# set(stopwords.words('english'))
    tokens = [token for token in tokens if token not in stop_words]

    # Stemming
    #stemmer = PorterStemmer()
    #tokens = [stemmer.stem(token) for token in tokens]

    # Remove special characters, digits, and HTML tags
    tokens = [token for token in tokens if token.isalpha()]

    return " ".join(tokens)
def preprocess_token(text):
    # Tokenize
    tokens = word_tokenize(text.lower())

    # Remove stopwords
    stop_words =en_stop# set(stopwords.words('english'))
    tokens = [token for token in tokens if token not in stop_words]

    # Stemming
    stemmer = PorterStemmer()
    tokens = [stemmer.stem(token) for token in tokens]

    # Remove special characters, digits, and HTML tags
    tokens = [token for token in tokens if token.isalpha()]

    return tokens

def getHashtag(tweet):
	# return list of hashtags
	hashtag_rx = re.compile(r'#\w+')
	hashtags = hashtag_rx.findall(tweet)
	return hashtags

def getAllHashtags(all_tweets):
	all_hashtags = []
	for tweet in all_tweets:
		hashtags_list = getHashtag(tweet)
		all_hashtags.extend(hashtags_list)
	return all_hashtags


def cleanTweetText(tweet):
	retweet_rx = re.compile(r'RT @\w+:\s')
	url_rx = re.compile(r'https://\w\.\w+/\w+')
	hashtag_rx = re.compile(r'#\w+\s')
	to_user_rx = re.compile(r'@\w+\s')
	regex      = [retweet_rx, url_rx, hashtag_rx, to_user_rx]
	for rx in regex:
		tweet = re.sub(rx, '', tweet)
		tweet=re.sub(r'[^\w\s#@/:%.,_-]', '', tweet)
	return  re.sub(r'\n\n', ' ', tweet)


from gensim.models import Word2Vec
from gensim.models.doc2vec import Doc2Vec, TaggedDocument
from top2vec import Top2Vec
from sklearn.feature_extraction.text import TfidfVectorizer
def Word2Vector(wordlist,vectorsize=5,windowsize=2,ignore_freq=1,method="cbow"):
    '''

    :param textlist: list of wordlist
    :param vectorsize:
    :param windowsize:
    :param ignore_freq:
    :param method: "cbow" or "skip-gram"
    :return:
    '''
    if method=="cbow":
        kk=0
    elif method=="skip-gram":
        kk=1
    return Word2Vec(wordlist, vector_size=vectorsize, window=windowsize, min_count=ignore_freq, sg=kk)


def Doc2Vector(wordlist,vectorsize=20,windowsize=2,ignore_freq=1,docvector_only=True,workers=4,epochs=100):
    '''

    :param wordlist:
    :param vectorsize:
    :param windowsize:
    :param ignore_freq:
    :param docvector_only:  – If set to False trains word-vectors (in skip-gram fashion) simultaneous with DBOW doc-vector training; If True, only trains doc-vectors (faster).
    :param workers:
    :param epochs:
    :return:
    '''
    if docvector_only==True:
        kk=0
    else:
        kk=1
    tagged_data = [TaggedDocument(words=doc, tags=[i]) for i, doc in enumerate(wordlist)]
    model = Doc2Vec(vector_size=vectorsize, window=windowsize, min_count=ignore_freq, dbow_words=kk, workers=workers, epochs=epochs)
    # Build the vocabulary
    model.build_vocab(tagged_data)

    # Train the model
    model.train(tagged_data, total_examples=model.corpus_count, epochs=model.epochs)
    doc_embeddings = [model.infer_vector(doc) for doc in wordlist]
    return model,tagged_data,doc_embeddings


def Topic2Vector(textlist,  doc2vec_model='universal-sentence-encoder',ignore_freq=1,speed="slow",workers=4):
    '''

    :param textlist: doc2vec,  universal-sentence-encoder ,universal-sentence-encoder-large, universal-sentence-encoder-multilingual,universal-sentence-encoder-multilingual-large, distiluse-base-multilingual-cased,all-MiniLM-L6-v2, paraphrase-multilingual-MiniLM-L12-v2
    :param doc2vec_model:
    :param ignore_freq:
    :param speed:
    :param workers:
    :return:
    '''
    if speed=="slow":
        speed="learn"
    model = Top2Vec(documents=textlist, embedding_model=doc2vec_model, min_count=ignore_freq, speed=speed,workers=workers)
    return model

def TF_IDF(sentence):
    vectorizer = TfidfVectorizer()
    return vectorizer.fit_transform(sentence).toarray()





from setuptools import setup, find_packages

setup(
    name='wansuite',
    version='3.2',
    description='A Simplified Toolkits for Wan ',
    packages=find_packages(),
    install_requires=[
       'yfinance',
'beautifulsoup4',
        'helium',
'mysql-connector-python',
'SQLAlchemy',
        'ib_insync',
        'nltk',
        'gensim',
        'nltk-downloader',
        'top2vec'



    ]
)





import numpy as np
import pandas as pd
import wansuite as ws
import yfinance as yf
import pandas as pd
from ib_insync import *
import numpy as np
import datetime as dt
import pytz
#%%
def updateBondIndexYahooDaily(date):
    '''
    update daily us stock price from health and computer sectors
    :param date: starting date
    :param interval:"1d", "1m"
    :return:
    '''
    pairlist = ["DX-Y.NYB","^IRX","^FVX","^TNX","^TYX","^GSPC","^IXIC","^VIX","^FTSE","^N225","^HSI","000001.SS","^AORD","^TWII","BTC-USD"]
    for n in pairlist:
        print(n)
        temp = yf.download(n, start=date, progress=False)
        temp["Symbol"] = n
        temp = temp.reset_index()
        ws.msql.df2table("market", temp,"yf_bondindex_1d", "append")
def updateFutureYahooDaily(date):
    '''
    update daily us stock price from health and computer sectors
    :param date: starting date
    :param interval:"1d", "1m"
    :return:
    '''
    pairlist = ws.msql.table2df("symbol", "future")["Symbol"].dropna()
    for n in pairlist:
        temp = yf.download(n, start=date, progress=False)
        temp["Symbol"] = n
        temp = temp.reset_index()
        ws.msql.df2table("market", temp,"yf_future_1d", "append")
def updateCurrencyYahooDaily(date):
    '''
    update daily us stock price from health and computer sectors
    :param date: starting date
    :param interval:"1d", "1m"
    :return:
    '''

    pairlist = ws.msql.table2df("symbol", "currencypair")["Ticker"].dropna()
    for n in pairlist:
        print("currency", n)
        temp = yf.download(n + "=X", start=date, progress=False)
        temp["Symbol"] = n

        temp = temp.reset_index()
        ws.msql.df2table("market", temp,"yf_currency_1d", "append")
def updateStockYahooDaily(date):
    '''
    update daily us stock price from health and computer sectors
    :param date: starting date
    :param interval:"1d", "1m"
    :return:
    '''
    sectorlist=["us_medical_sector","us_computer_sector","us_finance_sector","energy","service","products","discretionary","staples"]
    tablelist=["yf_health_1d","yf_computer_1d","yf_finance_1d","yf_energy_1d","yf_service_1d","yf_products_1d","yf_discretionary_1d","yf_staples_1d"]


    for h in range(len(sectorlist)):
        print("sector:",  sectorlist[h])
        sec= ws.msql.table2df("symbol", sectorlist[h])["Ticker"].dropna().to_list()

        for n in sec:
            temp = yf.download(n, start=date, progress=False)
            temp["Symbol"] = n
            temp = temp.reset_index()
            ws.msql.df2table("market", temp, tablelist[h], "append")


def updateBondIndexYahooMinute(date):
    '''
    update daily us stock price from health and computer sectors
    :param date: starting date
    :param interval:"1d", "1m"
    :return:
    '''

    pairlist = ["DX-Y.NYB","^IRX","^FVX","^TNX","^TYX","^GSPC","^IXIC","^VIX","^FTSE","^N225","^HSI","000001.SS","^AORD","^TWII","BTC-USD"]



    for n in pairlist:
        print("future",n)
        temp = yf.download(n, start=date,interval="1m", progress=False)
        temp["Symbol"] = n
        temp=temp.tz_convert("US/Eastern")
        temp = temp.reset_index()

        ws.msql.df2table("market", temp, "yf_bondinde_5m", "append")

def updateFutureYahooMinute(date):
    '''
    update daily us stock price from health and computer sectors
    :param date: starting date
    :param interval:"1d", "1m"
    :return:
    '''

    pairlist = ws.msql.table2df("symbol", "future")["Symbol"].dropna()


    for n in pairlist:
        print("future",n)
        temp = yf.download(n, start=date,interval="1m", progress=False)
        temp["Symbol"] = n
        temp=temp.tz_convert("US/Eastern")
        temp = temp.reset_index()

        ws.msql.df2table("market", temp, "yf_future_5m", "append")
def updateCurrencyYahooMinute(date):
    '''
    update daily us stock price from health and computer sectors
    :param date: starting date
    :param interval:"1d", "1m"
    :return:
    '''

    pairlist = ws.msql.table2df("symbol", "currencypair")["Ticker"].dropna()


    for n in pairlist:
        print("currency",n)
        temp = yf.download(n+"=X", start=date,interval="1m", progress=False)
        temp["Symbol"] = n
        temp=temp.tz_convert("US/Eastern")
        temp = temp.reset_index()

        ws.msql.df2table("market", temp, "yf_currency_5m", "append")




def updateStockYahooMinute(date):
    '''
    update daily us stock price from health and computer sectors
    :param date: starting date
    :param interval:"1d", "1m"
    :return:
    '''
    health = ws.msql.table2df("symbol", "us_medical_sector")["Ticker"].dropna().to_list()
    computer = ws.msql.table2df("symbol", "us_computer_sector")["Ticker"].dropna().to_list()
    sectorlist = ["us_medical_sector", "us_medical_sector", "us_finance_sector", "energy", "service", "products",
                  "discretionary", "staples"]
    tablelist = ["yf_health_5m", "yf_computer_5m", "yf_finance_5m", "yf_energy_5m", "yf_service_5m", "yf_products_5m",
                 "yf_discretionary_5m", "yf_staples_5m"]

    for n in range(len(sectorlist)):
        print("sector in 1 min", sectorlist[n])
        stocklist=ws.msql.table2df("symbol", sectorlist[n])["Ticker"].dropna().to_list()
        for m in stocklist:
            temp = yf.download(m, start=date, interval="1m", progress=False)
            temp["Symbol"] = m
            temp = temp.reset_index()
            ws.msql.df2table("market", temp, tablelist[n], "append")

def updateStockIB(enddate,duration,barsize,stocklist,database,table):
    '''

    :param enddate: "2024-01-01"
    :param duration: "30 D"
    :param barsize: "5 mins"
    :param stocklist:
    :param database: "market"
    :param table:sector_health_5m,sector_computer_5m
    :newtable: True
    :return:null
    '''
    #util.startLoop()
    ib = IB()
    ib.connect('127.0.0.1', 7496, clientId=np.random.randint(10000))

    flist = [Stock(x, "SMART", "USD") for x in stocklist]
    stday = enddate
    misslist = []
    for ti in [0]:
        ib.sleep(5)
        for n in range(len(stocklist)):  # #for stock in
            try:
                stock = stocklist[n]
                bars = ib.reqHistoricalData(flist[n],
                                            endDateTime=stday + " 19:00:00" + " US/Eastern",
                                            durationStr=duration,
                                            barSizeSetting=barsize,  # 1 day
                                            whatToShow="TRADES",  # TRADES
                                            useRTH=1,
                                            keepUpToDate=False,
                                            formatDate=1)
                if len(bars) == 0:
                    misslist.append(stock)
                if n % 50 == 0:
                    print(flist[n])
                df = pd.DataFrame(bars)
                df["symbol"] = stock

                ws.msql.df2table(database, df, table, "append")
            except:
                print("cannot download", stock)
                misslist.append(stock)

    return misslist


def updateCurrencyIB(enddate,duration,barsize,pairlist,database, table,newtable):
    '''

    :param enddate: "2024-01-01"
    :param duration: "30 D"
    :param barsize: "5 mins"
    :param pairlist:
    :param database: "market"
    :param table:sector_health_5m,sector_computer_5m
    :newtable: True
    :return:null
    '''
    #util.startLoop()
    ib = IB()
    ib.connect('127.0.0.1', 7496, clientId=np.random.randint(10000))

    flist =[Forex(x) for x in pairlist]
    stday = enddate
    misslist = []
    for ti in [0]:
        ib.sleep(5)
        for n in range(len(pairlist)):  # #for stock in
            try:
                stock = pairlist[n]
                bars = ib.reqHistoricalData(flist[n],
                                            endDateTime=stday + " 19:00:00" + " US/Eastern",
                                            durationStr=duration,
                                            barSizeSetting=barsize,
                                            whatToShow="MIDPOINT",  # TRADES
                                            keepUpToDate=False,
                                            useRTH=0,
                                            formatDate=1)
                if len(bars) == 0:
                    misslist.append(stock)
                if n % 50 == 0:
                    print(flist[n])
                df = pd.DataFrame(bars)
                df["symbol"] = stock
                if newtable==True:
                    ws.msql.df2table(database, df, table, "replace")
                else:
                    ws.msql.df2table(database, df, table, "append")
            except:
                print("cannot download", stock)
                misslist.append(stock)

    return misslist

def updateForward(asset):
    if asset=="currency":
        stock2 = downloadfromsql("market", "currency_5m", "date")
        ustoday = dt.datetime.now(pytz.timezone('US/Eastern')).date()
        print("Last date for currency", stock2.index[-1])
        print("today", dt.datetime.now(pytz.timezone('US/Eastern')).date())
        diff = (ustoday - stock2.index[-1].date()).days
        print(diff, " days need to be updated")
        if diff > 0:
            startdate = str((stock4.index[-1] + dt.timedelta(days=1)).date())

            pairlist = ws.msql.table2df("symbol", "currencypair")["Ticker"].dropna()
            misslist = updateCurrencyIB(temp, str(diff) + " D", "1 min", pairlist,
                                        "market", "currency_5m", False)

        print("Currency 1 Minute is updated from IBKR")
    elif asset=="yahoo":
        stock1 = downloadfromsql("market", "yf_computer_1d", "Date")
        ustoday = dt.datetime.now(pytz.timezone('US/Eastern')).date()
        print("today", dt.datetime.now(pytz.timezone('US/Eastern')).date())
        print("Last date for yahoo daily stock price", stock1.index[-1])
        diff = (ustoday - stock1.index[-1].date()).days
        print(diff, " days need to be updated")
        if diff > 0:
            startdate = str((stock4.index[-1] + dt.timedelta(days=1)).date())

            pairlist = ws.msql.table2df("symbol", "currencypair")["Ticker"].dropna()
            misslist = updateCurrencyIB(temp, str(diff) + " D", "1 min", pairlist,
                                        "market", "currency_5m", False)

        print("Currency 1 Minute is updated from IBKR")



import pandas as pd
import numpy as np
import wansuite as ws
import yfinance as yf
downloadcol=['Company Name', 'Ticker','today','Last Close', 'Market Cap (mil)', 'Exchange',
       'Month of Fiscal Yr End', 'Sector', 'Industry',
       'Shares Outstanding (mil)', 'Avg Volume',
       'Price as a % of 52 Wk H-L Range', '% Price Change (1 Week)',
       '% Price Change (4 Weeks)', '% Price Change (12 Weeks)',
       'Average Target Price', '% Rating Downgrades ', '% Rating Upgrades ',
       '% Rating Hold', '% Rating Strong Sell or Sell',
       '% Rating Strong Buy or Buy',
       'Q0 Consensus Est. (last completed fiscal Qtr)', 'Q1 Consensus Est. ',
       'Q2 Consensus Est. (next fiscal Qtr)', 'F0 Consensus Est.',
       'F1 Consensus Est.', 'F2 Consensus Est.', 'Current ROE (TTM)',
       'Current ROI (TTM)', 'ROI (5 Yr Avg)', 'Current ROA (TTM)',
       'ROA (5 Yr Avg)', 'Market Value/# Analysts', 'EBITDA ($mil)',
       'EBIT ($mil)', 'Net Income  ($mil)', 'Cash Flow ($mil)',
       'Net Income Growth F(0)/F(-1)', 'Net Margin %', 'Turnover',
       'Inventory Turnover', 'Asset Utilization', 'Operating Margin 12 Mo %',
       'Receivables ($mil)', 'Inventory ($mil)', 'Intangibles ($mil)',
       'Current Assets  ($mil)', 'Current Liabilities ($mil)',
       'Long Term Debt ($mil)', 'Preferred Equity ($mil)',
       'Common Equity ($mil)', 'Book Value', 'Debt/Total Capital',
       'Debt/Equity Ratio', 'Current Ratio', 'Quick Ratio', 'Cash Ratio',
       ]
changecol=['CompanyName', 'Ticker','Today','Price', 'MarketCap_mil', 'Exchange',
       'MonthofFiscalYrEnd', 'Sector', 'Industry',
       'SharesOutstanding_mil', 'AvgVolume',
       'Price_Percentof52WkH-LRange', 'Percent_PriceChange_1Week',
       'Percent_PriceChange_4Weeks', 'Percent_PriceChange_12Weeks',
       'AverageTargetPrice', 'Percent_RatingDowngrades', 'Percent_RatingUpgrades',
       'Percent_Rating_Hold', 'Percent_RatingStrongSellorSell',
       'Percent_Rating_StrongBuyorBuy',
       'Q0EPS', 'Q1EPS',
       'Q2EPS', 'F0EPS',
       'F1EPS', 'F2EPS', 'CurrentROE_TTM',
       'CurrentROI_TTM', 'ROI_ 5YrAvg', 'CurrentROA_TTM',
       'ROA_5YrAvg', 'MarketValue/Analysts', 'EBITDA_mil',
       'EBIT_mil', 'NetIncome_mil', 'CashFlow_mil',
       'NetIncomeGrowthF0/F-1', 'NetMargin_Percent', 'Turnover',
       'InventoryTurnover', 'AssetUtilization', 'OperatingMargin_12Mo_Percent',
       'Receivables_mil', 'Inventory_mil', 'Intangibles_mil',
       'CurrentAssets_mil', 'CurrentLiabilities_mil',
       'LongTermDebt_mil', 'PreferredEquity_mil',
       'CommonEquity_mil', 'BookValue', 'Debt/TotalCapital',
       'Debt/EquityRatio', 'CurrentRatio', 'QuickRatio', 'CashRatio',
       ]



dellist=["underlyingSymbol", "shortName","longName",'country',"currency" ,'sectorKey',
       'sectorDisp', 'longBusinessSummary','industryKey', 'industryDisp', 'website',"companyOfficers","address1","city","state","zip","financialCurrency","phone"]
def updateFundmental_Yahoo(today):
    sl=list(ws.msql.table2df("symbol","allstocks")["Ticker"].dropna())
    for symbol in sl:
           temp=yf.Ticker(symbol)
           data=pd.DataFrame(temp.info).drop(columns=dellist)
           data["date"]=today
           A=data.iloc[-1].to_frame().T
           ml=[]
           for c in A.columns[:]:
               if type(A.loc[A.index[0],c])==str:
                   ml.append(A.loc[A.index[0],c])
               else:
                   ml.append(float(A.loc[A.index[0],c]))
           A.loc[A.index[0]]=ml
           ws.msql.df2table("macro",A,"yf_fund","append")
def convertForexData(filename):
    data = pd.read_csv(filename, index_col=0)
    data = data.reset_index()
    date = data.columns[0]
    data = data.rename(columns={data.columns[0]: "time"})
    data = data[data.time != "All Day"]
    data = data[data["Unnamed: 2"] != "Holiday"]
    data = data[data["time"] != "Tentative"]
    data["date"] = np.nan
    data.isnull().sum(axis=1).sort_values(ascending=False)


    for n in data.index:
        if data.loc[n].isnull().sum() == 7:
            date = data.loc[n, "time"]
            # print("OK",data.loc[n,"time"])
        else:

            data.loc[n, "date"] = pd.to_datetime(date + " " + data.loc[n, "time"])

    data = data.drop(columns=["Unnamed: 2", "time"])
    data = data.dropna(subset=["date"])
    data = data.rename(
        columns={"Unnamed: 1": "Currency", "Unnamed: 3": "Indicator", "Unnamed: 4": "Actual", "Unnamed: 5": "Forecast",
                 "Unnamed: 6": "Previous"})
    droplist = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]

    indicator = []
    for i in data.Indicator:
        if i[-4:-1] in droplist:
            indicator.append(i[:-5])
        else:
            indicator.append(i)
    data["Indicator"] = indicator

    droplist = ["Q1", "Q2", "Q3", "Q4"]
    indicator = []
    for i in data.Indicator:
        if i[-3:-1] in droplist:
            indicator.append(i[:-4])
        else:
            indicator.append(i)
    data["Indicator"] = indicator

    return  data[data.isnull().sum(axis=1) < 3]

def updateCompanyData(filename,date):
    zack = pd.read_csv(filename)
    zack["today"] = date

    zack = zack[zack["Exchange"] != "OTCBB"]
    zack = zack[downloadcol]
    zack.columns = changecol


    return zack

def updateEarning(filename):
    data = pd.read_csv(filename)
    date = data.columns[0]
    data = data.rename(columns={data.columns[0]: "date"})
    for n in data.index:
        if data.loc[n].isnull().sum() == 6:
            date = data.loc[n, "date"]
            # print("OK",data.loc[n,"time"])
        else:
            data.loc[n, "date"] = pd.to_datetime(date)
    data = data.dropna(subset=["Unnamed: 1"])
    data = data.rename(columns={"Unnamed: 1": "Company", "Unnamed: 2": "Actual EPS", "Unnamed: 3": "Forcast EPS",
                                "Unnamed: 4": "Actual Revenue", "Unnamed: 5": "Forcast Revenue",
                                "Unnamed: 6": "Market Cap"})
    temp = []
    for t in data.index:
        i = data.loc[t, "Actual EPS"]
        if i == "--":
            temp.append(np.nan)
        else:
            if i[-1] == "K":
                temp.append(float(i[:-1].replace(",", "")) * 1000)
            else:
                temp.append(float(i.replace(",", "")))
    data["Actual EPS"] = temp
    temp = []
    for t in data["Forcast EPS"].index:
        i = data.loc[t, "Forcast EPS"]
        if i == "/--" or i == "/ --":
            temp.append(np.nan)
        else:
            # print(i)
            temp.append(float(i[1:].replace(",", "")))
    data[("Forcast EPS")] = temp
    temp = []
    for i in data["Actual Revenue"]:
        if i == "--":
            temp.append(np.nan)
        else:
            temp.append(i)
    data["Actual Revenue"] = temp
    temp = []
    for t in data["Forcast Revenue"].index:
        i = data.loc[t, "Forcast Revenue"]
        if i == "/--" or i == "/ --":
            temp.append(np.nan)
        else:
            # print(i)
            temp.append(i[1:])
    data[("Forcast Revenue")] = temp
    data["date"] = pd.to_datetime(data["date"])
    data["Symbol"] = np.nan
    temp = []
    for i in data.Company:
        if i[-3] == "(":
            temp.append(i[-2:-1])
        elif i[-4] == "(":
            temp.append(i[-3:-1])
        elif i[-5] == "(":
            temp.append(i[-4:-1])
        elif i[-6] == "(":
            temp.append(i[-5:-1])
        elif i[-7] == "(":
            temp.append(i[-6:-1])
        elif i[-8] == "(":
            temp.append(i[-7:-1])
        elif i[-9] == "(":
            temp.append(i[-8:-1])
        else:
            print(i)
    data["Symbol"] = temp
    data["date"] =[x.date() for x in  pd.to_datetime(data["date"])]
    return data

